/***************************************************************
 * Name:      wxiscanapp.cpp
 * Purpose:   Code for Application Class
 * Author:    Daniel Nell (daniel.nell@nellresearch.de)
 * Created:   2008-05-04
 * Copyright: Daniel Nell (www.nellresearch.de)
 * License:   GPL2
 **************************************************************/

#include "wx_pch.h"

//(*AppHeaders
#include "wxiscanframe.h"
#include <wx/image.h>
//*)

// wxWidgets headers
#include <wx/cmdline.h>
#include <wx/config.h>

// Private headers
#include "wxiscanapp.h"


IMPLEMENT_APP(wxIScanApp);

bool wxIScanApp::OnInit()
{
    // Get application configuration.
    wxConfigBase *poConfig = wxConfigBase::Get();

    poConfig->SetRecordDefaults();
    poConfig->SetPath( wxT( "/Global" ) );
    poConfig->Write( wxT( "App" ), argv[0] );

    // Set locale.
    if( poConfig->Read( wxT( "I18n" ), (long)true ) )
    {
//		// Add lookup path for subdirectories direct below the
//		// application's installation directory.
//		//
//		// NOTE: We do this because Un*x like OSes do not search
//		//       the application's directory automatically!
//		//
//		wxFileName oFileName( argv[0] );
//
//		oFileName.Normalize();
//		m_oLocale.AddCatalogLookupPathPrefix( oFileName.GetPath() );
//
		// Initialize locale.
        m_oLocale.Init( wxLocale::GetSystemLanguage() );

		// Add application catalog.
        m_oLocale.AddCatalog( GetAppName() );

		// Add standard wxWidgets translation catalog.
        m_oLocale.AddCatalog( wxT( "wxstd" ) );
    }


   // Install image handlers.
    AddImageHandler( new wxBMPHandler );
#if wxUSE_LIBPNG
    AddImageHandler( new wxPNGHandler );
#endif
#if wxUSE_LIBJPEG
    AddImageHandler( new wxJPEGHandler );
#endif
#if wxUSE_LIBTIFF
    AddImageHandler( new wxTIFFHandler );
#endif
#if wxUSE_GIF
    AddImageHandler( new wxGIFHandler );
#endif
#if wxUSE_PCX
    AddImageHandler( new wxPCXHandler );
#endif
#if wxUSE_PNM
    AddImageHandler( new wxPNMHandler );
#endif
#if wxUSE_XPM
    AddImageHandler( new wxXPMHandler );
#endif
#if wxUSE_ICO_CUR
    AddImageHandler( new wxICOHandler );
    AddImageHandler( new wxCURHandler );
    AddImageHandler( new wxANIHandler );
#endif


  	m_poFrame = new wxIScanFrame( 0 );
	if( m_poFrame == NULL )
    {
        ::wxLogFatalError( _( "Failed to create main window." ) );

        // Program termination.
        return false;
    }
    m_poFrame->Show();
    SetTopWindow( m_poFrame );

    // Call the base class's OnInit() function for commandline processing,
    // and do someerror checking.
    //
    // Note: A return value of 'false' will cause the application to terminate
    //       immediately!
    //
    if( !wxApp::OnInit() )
    {
        // Program termination.
        return false;
    }

    return true;
}

void wxIScanApp::OnInitCmdLine( wxCmdLineParser& oParser )
{
    // Call the base class's OnInitCmdLine function for default
    // parameter handling.
    wxApp::OnInitCmdLine( oParser );

    // Add an optional filename parameter to the parameter list.
    oParser.AddParam( wxT( "[Filename 1[ [... [Filename n]]]]" ),
                      wxCMD_LINE_VAL_STRING,
                      wxCMD_LINE_PARAM_OPTIONAL | wxCMD_LINE_PARAM_MULTIPLE );
}

bool wxIScanApp::OnCmdLineParsed( wxCmdLineParser& oParser )
{
    // Call the base class's OnCmdLineParsed function for default
    // parameter handling.
    if( !wxApp::OnCmdLineParsed( oParser ) )
    {
        // Return error.
        return false;
    }

    // Handle filename parameters by calling the top window's
    // FileOpen() function.
    int nCount= oParser.GetParamCount();

    for( int i= 0; i < nCount; i++ )
    {
        m_poFrame->FileOpen( oParser.GetParam( i ) );
    }

    // Return OK.
    return true;
}

void wxIScanApp::AddImageHandler( wxImageHandler* poHandler )
{
    if( poHandler )
    {
        wxString strExtension= poHandler->GetExtension();

		// TODO: Is there a posibility to get files by extension fully case
		//       insensitive (see void wxIViewFrame::ReadDir).
		//
#ifdef __WXMSW__
        m_strImageFilter += wxT( "*." ) + strExtension + wxT( ";" );
        m_strImageFilterList += poHandler->GetMimeType() + wxT( " (*." )
		                        + strExtension
                                + wxT( ")|*." ) + strExtension + wxT( "|" );
#else
        m_strImageFilter += wxT( "*." ) + strExtension.Lower() + wxT( ";" )
		                 +  wxT( "*." ) + strExtension.Upper() + wxT( ";" );
        m_strImageFilterList += poHandler->GetMimeType() + wxT( " (*." )
		                        + strExtension
                                + wxT( ")|*." )
								+ strExtension.Lower()
								+ wxT( ";*." )
								+ strExtension.Upper()
								+ wxT( "|" );
#endif
		if( wxImage::FindHandler( poHandler->GetName() ) )
		{
			delete poHandler;
		}
		else
		{
        	wxImage::AddHandler( poHandler );
		}
    }
}

wxString wxIScanApp::GetImageFilter()
{
    return _( "All image files|" )
           + m_strImageFilter + wxT( "|" )
           + m_strImageFilterList
           + _( "All files|*.*" );
}
