/*************************************************************
 * Name:      wxiscanframe.cpp
 * Purpose:   Code for Application Frame
 * Author:    Daniel Nell (daniel.nell@nellresearch.de)
 * Created:   2008-05-04
 * Copyright: Daniel Nell (www.nellresearch.de)
 * License:   GPL2
 **************************************************************/

// Include precompiled headers.
#include "wx_pch.h"

// wxWidgets headers
#include <wx/msgdlg.h>
#include <wx/aboutdlg.h>
#include <wx/config.h>
#include <wx/filename.h>
#if __WXPDFDOC__
    #include <wx/pdfdoc.h>
#endif // __WXPDFDOC__
#include <wx/print.h>
#include <wx/printdlg.h>
#if __DEBUG__
    #include <wx/stdpaths.h>
#endif // __DEBUG__
#include <wx/textfile.h>

//(*InternalHeaders(wxIScanFrame)
#include <wx/string.h>
#include <wx/intl.h>
#include <wx/bitmap.h>
#include <wx/image.h>
#include <wx/artprov.h>
//*)

// Private headers
#include "wxiscanapp.h"
#include "wxiscanframe.h"
#include "wxiscanimageprofile.h"
#include "wxiscanresources.h"
#include "wxiscan.h"
#include "wxscan.h"
#include "wxchainlogtextctrl.h"
#include "wxiviewprintout.h"

// Resources
//}

//(*IdInit(wxIScanFrame)
const long wxIScanFrame::ID_CANVAS = wxNewId();
const long wxIScanFrame::ID_LOGTEXTCTRL = wxNewId();
const long wxIScanFrame::ID_HSPLITTERWINDOW = wxNewId();
const long wxIScanFrame::ID_FILESCANIMAGE = wxNewId();
const long wxIScanFrame::ID_FILEOPEN = wxNewId();
const long wxIScanFrame::ID_FILESAVEAS = wxNewId();
const long wxIScanFrame::ID_FILEPRINT = wxNewId();
const long wxIScanFrame::ID_FILEPRINTPREVIEW = wxNewId();
const long wxIScanFrame::ID_FILEPRINTSETUP = wxNewId();
const long wxIScanFrame::ID_FILEQUIT = wxNewId();
const long wxIScanFrame::ID_DOCUMENTFIRSTPAGE = wxNewId();
const long wxIScanFrame::ID_DOCUMENTPREVIOUSPAGE = wxNewId();
const long wxIScanFrame::ID_DOCUMENTNEXTPAGE = wxNewId();
const long wxIScanFrame::ID_DOCUMENTLASTPAGE = wxNewId();
const long wxIScanFrame::ID_DOCUMENTROTATE90LEFT = wxNewId();
const long wxIScanFrame::ID_DOCUMENTROTATE90RIGHT = wxNewId();
const long wxIScanFrame::ID_DOCUMENTAPPLYPROFILE = wxNewId();
const long wxIScanFrame::ID_MENUDOCUMENTPROFILES = wxNewId();
const long wxIScanFrame::ID_PDFBEGINPDF = wxNewId();
const long wxIScanFrame::ID_PDFADDPAGE = wxNewId();
const long wxIScanFrame::ID_PDFSAVEASPDF = wxNewId();
const long wxIScanFrame::ID_HELPABOUT = wxNewId();
const long wxIScanFrame::ID_STATUSBAR = wxNewId();
const long wxIScanFrame::ID_FILESCANIMAGE_TB = wxNewId();
const long wxIScanFrame::ID_FILEOPEN_TB = wxNewId();
const long wxIScanFrame::ID_FILESAVEAS_TB = wxNewId();
const long wxIScanFrame::ID_PDFBEGINPDF_TB = wxNewId();
const long wxIScanFrame::ID_PDFADDPAGE_TB = wxNewId();
const long wxIScanFrame::ID_PDFSAVEASPDF_TB = wxNewId();
const long wxIScanFrame::ID_DOCUMENTFIRSTPAGE_TB = wxNewId();
const long wxIScanFrame::ID_DOCUMENTPREVIOUSPAGE_TB = wxNewId();
const long wxIScanFrame::ID_DOCUMENTNEXTPAGE_TB = wxNewId();
const long wxIScanFrame::ID_DOCUMENTLASTPAGE_TB = wxNewId();
const long wxIScanFrame::ID_FILEPRINT_TB = wxNewId();
const long wxIScanFrame::ID_TOOLBAR = wxNewId();
//*)
const long wxIScanFrame::ID_DOCUMENTPROFILE0= wxNewId();


//////////////////////////////////////////////////////////
// Strings and other (global) constants
//
static const wxString cstrAppName=            wxT( PRODUCT_NAME );
static const wxString cstrAppDescription=     wxT( FILE_DESCRIPTION );
static const wxString cstrVendor=             wxT( COMPANY_NAME );
static const wxString cstrStatusCopyRight=    wxT( LEGAL_COPYRIGHT );
static const wxString cstrVersionString=      wxString::Format( wxT( "%d.%d.%d (build %d)" ),
                                                                AutoVersion::MAJOR,
                                                                AutoVersion::MINOR,
                                                                AutoVersion::BUILD,
                                                                AutoVersion::BUILDS_COUNT );
static const wxString cstrWebSite=            wxT( WEB_SITE );
static const wxString cstrwxFrame=            wxT( "wxIScanFrame" );


BEGIN_EVENT_TABLE( wxIScanFrame, wxFrame )
    //(*EventTable(wxIScanFrame)
    //*)
    // File history handling.
    EVT_MENU_RANGE( wxID_FILE1, wxID_FILE9, wxIScanFrame::OnMRUFiles )
    EVT_MENU_RANGE( ID_DOCUMENTPROFILE0, ID_DOCUMENTPROFILE0 + 999, wxIScanFrame::OnMRUFiles )
    // File drop event
    EVT_EXTFILEDROP( wxIScanFrame::OnFileDrop )
    // Handle canvas size changes.
    EVT_IVIEWCANVASSIZE( wxIScanFrame::OnCanvasSize )
END_EVENT_TABLE()

wxIScanFrame::wxIScanFrame( wxWindow* parent, wxWindowID id )
  : m_poPdfDoc( NULL ), m_nPdfPages( 0 ), m_bPdfDirty( false )
{
    //(*Initialize(wxIScanFrame)
    wxMenu* poMenuHelp;
    wxToolBarToolBase* ToolBarItem5;
    wxMenuItem* poMenuSavePdf;
    wxToolBarToolBase* ToolBarItem11;
    wxMenuItem* poDocumentRotate90Left;
    wxMenuItem* poMenuFileOpen;
    wxMenuItem* poMenuFileSaveAs;
    wxToolBarToolBase* ToolBarItem6;
    wxMenuItem* poHelpAbout;
    wxMenuItem* poFileQuit;
    wxMenuItem* poMenuPdfBeginPdf;
    wxMenuItem* poMenuFileScanImage;
    wxToolBarToolBase* ToolBarItem7;
    wxToolBarToolBase* ToolBarItem2;
    wxToolBarToolBase* ToolBarItem9;
    wxMenuItem* poMenuDocumentPreviousPage;
    wxMenuItem* poMenuFilePrint;
    wxMenu* poMenuDocument;
    wxMenuItem* poMenuFilePrintPreview;
    wxToolBarToolBase* ToolBarItem10;
    wxMenuItem* poMenuDocumentLastPage;
    wxMenuBar* m_poMenuBar;
    wxToolBarToolBase* ToolBarItem4;
    wxMenuItem* poMenuPdfAddPage;
    wxToolBarToolBase* ToolBarItem1;
    wxMenu* poMenuDocumentProfiles;
    wxMenuItem* poDocumentApplyProfile;
    wxToolBarToolBase* ToolBarItem3;
    wxMenu* poMenuFile;
    wxMenu* poMenuPdf;
    wxMenuItem* poMenuFilePrintSetup;
    wxToolBarToolBase* ToolBarItem8;
    wxMenuItem* poDocumentRotate90Right;
    wxMenuItem* poMenuDocumentFirstPage;
    wxMenuItem* poMenuDocumentNextPage;

    Create(parent, wxID_ANY, _("wxIScan"), wxDefaultPosition, wxDefaultSize, wxDEFAULT_FRAME_STYLE, _T("wxID_ANY"));
    m_poHSplitterWindow = new wxSplitterWindow(this, ID_HSPLITTERWINDOW, wxDefaultPosition, wxDefaultSize, wxSP_3D|wxSP_LIVE_UPDATE, _T("ID_HSPLITTERWINDOW"));
    m_poHSplitterWindow->SetMinSize(wxSize(10,10));
    m_poHSplitterWindow->SetMinimumPaneSize(10);
    m_poCanvas = new wxIViewCanvas(m_poHSplitterWindow, ID_CANVAS, wxDefaultPosition, wxDefaultSize, wxHSCROLL|wxVSCROLL, _T("ID_CANVAS"));
    m_poLogTextCtrl = new wxTextCtrl(m_poHSplitterWindow, ID_LOGTEXTCTRL, wxEmptyString, wxDefaultPosition, wxDefaultSize, wxTE_MULTILINE|wxTE_READONLY|wxTE_RICH|wxTE_AUTO_URL|wxTE_DONTWRAP, wxDefaultValidator, _T("ID_LOGTEXTCTRL"));
    m_poHSplitterWindow->SplitHorizontally(m_poCanvas, m_poLogTextCtrl);
    m_poHSplitterWindow->SetSashPosition(250);
    m_poMenuBar = new wxMenuBar();
    poMenuFile = new wxMenu();
    poMenuFileScanImage = new wxMenuItem(poMenuFile, ID_FILESCANIMAGE, _("Scan &Image\tCtrl+I"), _("Scan an image"), wxITEM_NORMAL);
    poMenuFileScanImage->SetBitmap(wxArtProvider::GetBitmap(wxART_MAKE_ART_ID_FROM_STR(_T("wxART_MISSING_IMAGE")),wxART_MENU));
    poMenuFile->Append(poMenuFileScanImage);
    poMenuFile->AppendSeparator();
    poMenuFileOpen = new wxMenuItem(poMenuFile, ID_FILEOPEN, _("&Open...\tCtrl+O"), _("Open an image file."), wxITEM_NORMAL);
    poMenuFileOpen->SetBitmap(wxArtProvider::GetBitmap(wxART_MAKE_ART_ID_FROM_STR(_T("wxART_FILE_OPEN")),wxART_MENU));
    poMenuFile->Append(poMenuFileOpen);
    poMenuFileSaveAs = new wxMenuItem(poMenuFile, ID_FILESAVEAS, _("Save &as...\tCtrl+S"), _("Save scanned image to file."), wxITEM_NORMAL);
    poMenuFileSaveAs->SetBitmap(wxArtProvider::GetBitmap(wxART_MAKE_ART_ID_FROM_STR(_T("wxART_FILE_SAVE_AS")),wxART_MENU));
    poMenuFile->Append(poMenuFileSaveAs);
    poMenuFileSaveAs->Enable(false);
    poMenuFile->AppendSeparator();
    poMenuFilePrint = new wxMenuItem(poMenuFile, ID_FILEPRINT, _("&Print...\tCtrl+P"), _("Print the current image."), wxITEM_NORMAL);
    poMenuFilePrint->SetBitmap(wxArtProvider::GetBitmap(wxART_MAKE_ART_ID_FROM_STR(_T("wxART_PRINT")),wxART_MENU));
    poMenuFile->Append(poMenuFilePrint);
    poMenuFilePrint->Enable(false);
    poMenuFilePrintPreview = new wxMenuItem(poMenuFile, ID_FILEPRINTPREVIEW, _("Print pre&view\tCtrl+Alt+P"), _("Show a print preview of the current image."), wxITEM_NORMAL);
    poMenuFile->Append(poMenuFilePrintPreview);
    poMenuFilePrintPreview->Enable(false);
    poMenuFilePrintPreview= poMenuFile->Remove( ID_FILEPRINTPREVIEW );
    poMenuFilePrintPreview->SetBitmap( fileprintpreviewbmp_xpm );
    poMenuFile->Append( poMenuFilePrintPreview );
    poMenuFilePrintSetup = new wxMenuItem(poMenuFile, ID_FILEPRINTSETUP, _("Page setup"), _("Show page setup dialog."), wxITEM_NORMAL);
    poMenuFile->Append(poMenuFilePrintSetup);
    poMenuFilePrintSetup= poMenuFile->Remove( ID_FILEPRINTSETUP );
    poMenuFilePrintSetup->SetBitmap( fileprintsetupbmp_xpm );
    poMenuFile->Append( poMenuFilePrintSetup );
    poMenuFile->AppendSeparator();
    poFileQuit = new wxMenuItem(poMenuFile, ID_FILEQUIT, _("&Quit\tCtrl+Q"), _("Quit the application"), wxITEM_NORMAL);
    poFileQuit->SetBitmap(wxArtProvider::GetBitmap(wxART_MAKE_ART_ID_FROM_STR(_T("wxART_QUIT")),wxART_MENU));
    poMenuFile->Append(poFileQuit);
    m_poMenuBar->Append(poMenuFile, _("&File"));
    poMenuDocument = new wxMenu();
    poMenuDocumentFirstPage = new wxMenuItem(poMenuDocument, ID_DOCUMENTFIRSTPAGE, _("&First page\tCtrl+Home"), _("Go to first document page."), wxITEM_NORMAL);
    poMenuDocumentFirstPage->SetBitmap(wxArtProvider::GetBitmap(wxART_MAKE_ART_ID_FROM_STR(_T("wxART_GO_BACK")),wxART_MENU));
    poMenuDocument->Append(poMenuDocumentFirstPage);
    poMenuDocumentFirstPage->Enable(false);
    poMenuDocumentPreviousPage = new wxMenuItem(poMenuDocument, ID_DOCUMENTPREVIOUSPAGE, _("&Previous page\tCtrl+PageUp"), _("Go to previous document page."), wxITEM_NORMAL);
    poMenuDocumentPreviousPage->SetBitmap(wxArtProvider::GetBitmap(wxART_MAKE_ART_ID_FROM_STR(_T("wxART_GO_BACK")),wxART_MENU));
    poMenuDocument->Append(poMenuDocumentPreviousPage);
    poMenuDocumentPreviousPage->Enable(false);
    poMenuDocumentNextPage = new wxMenuItem(poMenuDocument, ID_DOCUMENTNEXTPAGE, _("&Next page\tCtrl+PageDown"), _("Go to next document page."), wxITEM_NORMAL);
    poMenuDocumentNextPage->SetBitmap(wxArtProvider::GetBitmap(wxART_MAKE_ART_ID_FROM_STR(_T("wxART_GO_FORWARD")),wxART_MENU));
    poMenuDocument->Append(poMenuDocumentNextPage);
    poMenuDocumentNextPage->Enable(false);
    poMenuDocumentLastPage = new wxMenuItem(poMenuDocument, ID_DOCUMENTLASTPAGE, _("&Last page\tCtrl+End"), _("Go to last document page."), wxITEM_NORMAL);
    poMenuDocumentLastPage->SetBitmap(wxArtProvider::GetBitmap(wxART_MAKE_ART_ID_FROM_STR(_T("wxART_GO_FORWARD")),wxART_MENU));
    poMenuDocument->Append(poMenuDocumentLastPage);
    poMenuDocumentLastPage->Enable(false);
    poMenuDocument->AppendSeparator();
    poDocumentRotate90Left = new wxMenuItem(poMenuDocument, ID_DOCUMENTROTATE90LEFT, _("Rotate 90 degress left\tL"), _("Rotate 90 degress left"), wxITEM_NORMAL);
    poDocumentRotate90Left->SetBitmap(wxArtProvider::GetBitmap(wxART_MAKE_ART_ID_FROM_STR(_T("wxART_UNDO")),wxART_MENU));
    poMenuDocument->Append(poDocumentRotate90Left);
    poDocumentRotate90Left->Enable(false);
    poDocumentRotate90Right = new wxMenuItem(poMenuDocument, ID_DOCUMENTROTATE90RIGHT, _("Rotate 90 degress right\tR"), _("Rotate 90 degress right"), wxITEM_NORMAL);
    poDocumentRotate90Right->SetBitmap(wxArtProvider::GetBitmap(wxART_MAKE_ART_ID_FROM_STR(_T("wxART_REDO")),wxART_MENU));
    poMenuDocument->Append(poDocumentRotate90Right);
    poDocumentRotate90Right->Enable(false);
    poMenuDocument->AppendSeparator();
    poDocumentApplyProfile = new wxMenuItem(poMenuDocument, ID_DOCUMENTAPPLYPROFILE, _("&Apply profile\tp"), _("Apply the current selected image profile to the image."), wxITEM_NORMAL);
    poMenuDocument->Append(poDocumentApplyProfile);
    poDocumentApplyProfile->Enable(false);
    poMenuDocumentProfiles = new wxMenu();
    poMenuDocumentProfiles->AppendSeparator();
    poMenuDocument->Append(ID_MENUDOCUMENTPROFILES, _("&Profiles"), poMenuDocumentProfiles, wxEmptyString);
    m_poMenuBar->Append(poMenuDocument, _("&Document"));
    poMenuPdf = new wxMenu();
    poMenuPdfBeginPdf = new wxMenuItem(poMenuPdf, ID_PDFBEGINPDF, _("&Begin PDF\tCtrl+Alt+N"), _("Start a new PDF document."), wxITEM_NORMAL);
    poMenuPdfBeginPdf->SetBitmap(wxArtProvider::GetBitmap(wxART_MAKE_ART_ID_FROM_STR(_T("wxART_NEW")),wxART_MENU));
    poMenuPdf->Append(poMenuPdfBeginPdf);
    poMenuPdfBeginPdf->Enable(false);
    poMenuPdfAddPage = new wxMenuItem(poMenuPdf, ID_PDFADDPAGE, _("&Add Page\tCtrl++"), _("Add image as new page to the PDF document."), wxITEM_NORMAL);
    poMenuPdfAddPage->SetBitmap(wxArtProvider::GetBitmap(wxART_MAKE_ART_ID_FROM_STR(_T("wxART_ADD_BOOKMARK")),wxART_MENU));
    poMenuPdf->Append(poMenuPdfAddPage);
    poMenuPdfAddPage->Enable(false);
    poMenuPdf->AppendSeparator();
    poMenuSavePdf = new wxMenuItem(poMenuPdf, ID_PDFSAVEASPDF, _("Save &PDF...\tCtrl+Alt+S"), _("Save image as PDF file."), wxITEM_NORMAL);
    poMenuSavePdf->SetBitmap(wxArtProvider::GetBitmap(wxART_MAKE_ART_ID_FROM_STR(_T("wxART_FILE_SAVE")),wxART_MENU));
    poMenuPdf->Append(poMenuSavePdf);
    poMenuSavePdf->Enable(false);
    m_poMenuBar->Append(poMenuPdf, _("&PDF"));
    poMenuHelp = new wxMenu();
    poHelpAbout = new wxMenuItem(poMenuHelp, ID_HELPABOUT, _("&About\tCtrl-Alt-I"), _("Show info about this application"), wxITEM_NORMAL);
    poHelpAbout->SetBitmap(wxArtProvider::GetBitmap(wxART_MAKE_ART_ID_FROM_STR(_T("wxART_INFORMATION")),wxART_MENU));
    poMenuHelp->Append(poHelpAbout);
    m_poMenuBar->Append(poMenuHelp, _("&Help"));
    SetMenuBar(m_poMenuBar);
    m_poStatusBar = new wxStatusBar(this, ID_STATUSBAR, 0, _T("ID_STATUSBAR"));
    int __wxStatusBarWidths_1[3] = { -3, -1, -2 };
    int __wxStatusBarStyles_1[3] = { wxSB_NORMAL, wxSB_NORMAL, wxSB_NORMAL };
    m_poStatusBar->SetFieldsCount(3,__wxStatusBarWidths_1);
    m_poStatusBar->SetStatusStyles(3,__wxStatusBarStyles_1);
    SetStatusBar(m_poStatusBar);
    m_poToolBar = new wxToolBar(this, ID_TOOLBAR, wxDefaultPosition, wxDefaultSize, wxTB_VERTICAL|wxNO_BORDER, _T("ID_TOOLBAR"));
    ToolBarItem1 = m_poToolBar->AddTool(ID_FILESCANIMAGE_TB, _("Scan"), wxArtProvider::GetBitmap(wxART_MAKE_ART_ID_FROM_STR(_T("wxART_MISSING_IMAGE")),wxART_TOOLBAR), wxNullBitmap, wxITEM_NORMAL, _("Scan"), _("Scan an image"));
    ToolBarItem2 = m_poToolBar->AddTool(ID_FILEOPEN_TB, _("Open"), wxArtProvider::GetBitmap(wxART_MAKE_ART_ID_FROM_STR(_T("wxART_FILE_OPEN")),wxART_TOOLBAR), wxNullBitmap, wxITEM_NORMAL, _("Open file"), _("Open an image file."));
    ToolBarItem3 = m_poToolBar->AddTool(ID_FILESAVEAS_TB, _("Save as"), wxArtProvider::GetBitmap(wxART_MAKE_ART_ID_FROM_STR(_T("wxART_FILE_SAVE_AS")),wxART_TOOLBAR), wxNullBitmap, wxITEM_NORMAL, _("Save as"), _("Save scanned image to file."));
    m_poToolBar->AddSeparator();
    ToolBarItem4 = m_poToolBar->AddTool(ID_PDFBEGINPDF_TB, _("Begin PDF"), wxArtProvider::GetBitmap(wxART_MAKE_ART_ID_FROM_STR(_T("wxART_NEW")),wxART_TOOLBAR), wxNullBitmap, wxITEM_NORMAL, _("Begin PDF"), _("Start a new PDF document."));
    ToolBarItem5 = m_poToolBar->AddTool(ID_PDFADDPAGE_TB, _("Add Page"), wxArtProvider::GetBitmap(wxART_MAKE_ART_ID_FROM_STR(_T("wxART_ADD_BOOKMARK")),wxART_TOOLBAR), wxNullBitmap, wxITEM_NORMAL, _("Add PDF Page"), _("Add image as new page to the PDF document."));
    ToolBarItem6 = m_poToolBar->AddTool(ID_PDFSAVEASPDF_TB, _("Save PDF"), wxArtProvider::GetBitmap(wxART_MAKE_ART_ID_FROM_STR(_T("wxART_FILE_SAVE_AS")),wxART_TOOLBAR), wxNullBitmap, wxITEM_NORMAL, _("Save PDF"), _("Save image as PDF file."));
    m_poToolBar->AddSeparator();
    ToolBarItem7 = m_poToolBar->AddTool(ID_DOCUMENTFIRSTPAGE_TB, _("First page"), wxArtProvider::GetBitmap(wxART_MAKE_ART_ID_FROM_STR(_T("wxART_GO_BACK")),wxART_TOOLBAR), wxNullBitmap, wxITEM_NORMAL, _("First page"), _("Go To first document page."));
    ToolBarItem8 = m_poToolBar->AddTool(ID_DOCUMENTPREVIOUSPAGE_TB, _("Previous page"), wxArtProvider::GetBitmap(wxART_MAKE_ART_ID_FROM_STR(_T("wxART_GO_BACK")),wxART_TOOLBAR), wxNullBitmap, wxITEM_NORMAL, _("Previous page"), _("Go to previous document page."));
    ToolBarItem9 = m_poToolBar->AddTool(ID_DOCUMENTNEXTPAGE_TB, _("Next page"), wxArtProvider::GetBitmap(wxART_MAKE_ART_ID_FROM_STR(_T("wxART_GO_FORWARD")),wxART_TOOLBAR), wxNullBitmap, wxITEM_NORMAL, _("Next page"), _("Go to next document page."));
    ToolBarItem10 = m_poToolBar->AddTool(ID_DOCUMENTLASTPAGE_TB, _("Last page"), wxArtProvider::GetBitmap(wxART_MAKE_ART_ID_FROM_STR(_T("wxART_GO_FORWARD")),wxART_TOOLBAR), wxNullBitmap, wxITEM_NORMAL, _("Last page"), _("Go to last document page."));
    m_poToolBar->AddSeparator();
    ToolBarItem11 = m_poToolBar->AddTool(ID_FILEPRINT_TB, _("Print"), wxArtProvider::GetBitmap(wxART_MAKE_ART_ID_FROM_STR(_T("wxART_PRINT")),wxART_TOOLBAR), wxNullBitmap, wxITEM_NORMAL, _("Print"), _("Print the current image."));
    m_poToolBar->Realize();
    SetToolBar(m_poToolBar);

    Connect(ID_FILESCANIMAGE,wxEVT_COMMAND_MENU_SELECTED,(wxObjectEventFunction)&wxIScanFrame::OnFileScanImage);
    Connect(ID_FILEOPEN,wxEVT_COMMAND_MENU_SELECTED,(wxObjectEventFunction)&wxIScanFrame::OnFileOpen);
    Connect(ID_FILESAVEAS,wxEVT_COMMAND_MENU_SELECTED,(wxObjectEventFunction)&wxIScanFrame::OnFileSaveAs);
    Connect(ID_FILEPRINT,wxEVT_COMMAND_MENU_SELECTED,(wxObjectEventFunction)&wxIScanFrame::OnFilePrint);
    Connect(ID_FILEPRINTPREVIEW,wxEVT_COMMAND_MENU_SELECTED,(wxObjectEventFunction)&wxIScanFrame::OnFilePrintPreview);
    Connect(ID_FILEPRINTSETUP,wxEVT_COMMAND_MENU_SELECTED,(wxObjectEventFunction)&wxIScanFrame::OnFilePrintSetup);
    Connect(ID_FILEQUIT,wxEVT_COMMAND_MENU_SELECTED,(wxObjectEventFunction)&wxIScanFrame::OnFileQuit);
    Connect(ID_DOCUMENTFIRSTPAGE,wxEVT_COMMAND_MENU_SELECTED,(wxObjectEventFunction)&wxIScanFrame::OnDocumentFirstPage);
    Connect(ID_DOCUMENTPREVIOUSPAGE,wxEVT_COMMAND_MENU_SELECTED,(wxObjectEventFunction)&wxIScanFrame::OnDocumentPreviousPage);
    Connect(ID_DOCUMENTNEXTPAGE,wxEVT_COMMAND_MENU_SELECTED,(wxObjectEventFunction)&wxIScanFrame::OnDocumentNextPage);
    Connect(ID_DOCUMENTLASTPAGE,wxEVT_COMMAND_MENU_SELECTED,(wxObjectEventFunction)&wxIScanFrame::OnDocumentLastPage);
    Connect(ID_DOCUMENTROTATE90LEFT,wxEVT_COMMAND_MENU_SELECTED,(wxObjectEventFunction)&wxIScanFrame::OnDocumentRotate90Left);
    Connect(ID_DOCUMENTROTATE90RIGHT,wxEVT_COMMAND_MENU_SELECTED,(wxObjectEventFunction)&wxIScanFrame::OnDocumentRotate90Right);
    Connect(ID_DOCUMENTAPPLYPROFILE,wxEVT_COMMAND_MENU_SELECTED,(wxObjectEventFunction)&wxIScanFrame::OnDocumentApplyProfile);
    Connect(ID_PDFBEGINPDF,wxEVT_COMMAND_MENU_SELECTED,(wxObjectEventFunction)&wxIScanFrame::OnPdfBeginPdf);
    Connect(ID_PDFADDPAGE,wxEVT_COMMAND_MENU_SELECTED,(wxObjectEventFunction)&wxIScanFrame::OnPdfAddPage);
    Connect(ID_PDFSAVEASPDF,wxEVT_COMMAND_MENU_SELECTED,(wxObjectEventFunction)&wxIScanFrame::OnPdfSavePdf);
    Connect(ID_HELPABOUT,wxEVT_COMMAND_MENU_SELECTED,(wxObjectEventFunction)&wxIScanFrame::OnHelpAbout);
    Connect(ID_FILESCANIMAGE_TB,wxEVT_COMMAND_TOOL_CLICKED,(wxObjectEventFunction)&wxIScanFrame::OnFileScanImage);
    Connect(ID_FILEOPEN_TB,wxEVT_COMMAND_TOOL_CLICKED,(wxObjectEventFunction)&wxIScanFrame::OnFileOpen);
    Connect(ID_FILESAVEAS_TB,wxEVT_COMMAND_TOOL_CLICKED,(wxObjectEventFunction)&wxIScanFrame::OnFileSaveAs);
    Connect(ID_PDFBEGINPDF_TB,wxEVT_COMMAND_TOOL_CLICKED,(wxObjectEventFunction)&wxIScanFrame::OnPdfBeginPdf);
    Connect(ID_PDFADDPAGE_TB,wxEVT_COMMAND_TOOL_CLICKED,(wxObjectEventFunction)&wxIScanFrame::OnPdfAddPage);
    Connect(ID_PDFSAVEASPDF_TB,wxEVT_COMMAND_TOOL_CLICKED,(wxObjectEventFunction)&wxIScanFrame::OnPdfSavePdf);
    Connect(ID_DOCUMENTFIRSTPAGE_TB,wxEVT_COMMAND_TOOL_CLICKED,(wxObjectEventFunction)&wxIScanFrame::OnDocumentFirstPage);
    Connect(ID_DOCUMENTPREVIOUSPAGE_TB,wxEVT_COMMAND_TOOL_CLICKED,(wxObjectEventFunction)&wxIScanFrame::OnDocumentPreviousPage);
    Connect(ID_DOCUMENTNEXTPAGE_TB,wxEVT_COMMAND_TOOL_CLICKED,(wxObjectEventFunction)&wxIScanFrame::OnDocumentNextPage);
    Connect(ID_DOCUMENTLASTPAGE_TB,wxEVT_COMMAND_TOOL_CLICKED,(wxObjectEventFunction)&wxIScanFrame::OnDocumentLastPage);
    Connect(ID_FILEPRINT_TB,wxEVT_COMMAND_TOOL_CLICKED,(wxObjectEventFunction)&wxIScanFrame::OnFilePrint);
    Connect(wxID_ANY,wxEVT_CLOSE_WINDOW,(wxObjectEventFunction)&wxIScanFrame::OnClose);
    //*)

    // Custom Initializations:
    //
    m_nProfileIndex= 0;
    m_bPortrait= true;

    // Reset the image count, the page number and the image filename.
    m_nImageCount= 0;
    m_nPage= 0;
    m_strImageFileName= _( "unknown.jpg" );

#if defined( __WXPDFDOC__ ) && __WXPDFDOC__ >= 1
    if( !CreateNewPdfDocument() )
    {
        ::wxLogFatalError( _( "Failed to create wxPdfDocument object." ) );
    }
#endif

    // Reset window settings.
    wxConfigBase *poConfig = wxConfigBase::Get();

    poConfig->SetRecordDefaults();
    poConfig->SetPath( wxT( "/" )  );
    poConfig->SetPath( cstrwxFrame );

    int x= poConfig->Read( wxT( "x" ),   5 );
    int y= poConfig->Read( wxT( "y" ),   5 );
    int w= poConfig->Read( wxT( "w" ), 630 );
    int h= poConfig->Read( wxT( "h" ), 470 );
    bool bMaximized= (bool)poConfig->Read( wxT( "Maximized"  ), (long)false );

    // Reset window position, size and maximization state.
    Maximize( bMaximized );
    Move( x, y );
    SetSize( w, h );

    // Complete status bar information.
    SetStatusText( cstrStatusCopyRight, 2 );

    // Give the window an icon.
    SetIcon( wxiscanico_xpm );

    // Reset sash position and splitting option.
    int nHSashPos= poConfig->Read( wxT( "HSashPos" ), h / 5 );

    m_poHSplitterWindow->SetSashPosition( nHSashPos );
//    m_poHSplitterWindow->SetSashGravity( 1.0 );

    // ...
    m_nFit= poConfig->Read( wxT( "Fit" ), wxTIXFITWINDOW );

    // Get OCR configuration.
    m_bOCR= (bool)poConfig->Read( wxT( "OCR" ), (long)false );
    m_strCmdConvert= poConfig->Read( wxT( "CmdConvert" ), wxT( "convert %s -compress Group4 %s.tif" ) );
    m_strCmdOcr= poConfig->Read( wxT( "CmdOcr" ), wxT( "tesseract %s.tif %s" ) );
    m_strCmdCleanUp= poConfig->Read( wxT( "CmdCleanUp" ), wxT( "rm %s.tif" ) );


    // ...
#if defined( __WXPDFDOC__ ) && __WXPDFDOC__ >= 1
    GetMenuBar()->Enable(     ID_PDFBEGINPDF,    true  );
#else
    GetToolBar()->EnableTool( ID_PDFBEGINPDF_TB, false );
#endif

    // Switch (mostly disable) controls
    EnableControls();

    // Set drop target.
    SetDropTarget( new wxExtFileDropTarget( this ) );

    // Set style attributes to log text control:
    //
    // Read the font size from config file, ...
    int nFontSize= poConfig->Read( wxT( "FontSize" ), 8 );

    // ... get default text style parameters, and ...
    wxTextAttr oStyle( m_poLogTextCtrl->GetDefaultStyle() );

    // ... set some styling options (e. g. font).
    wxFont oFont( nFontSize, wxFONTFAMILY_MODERN, wxFONTSTYLE_NORMAL, wxFONTWEIGHT_NORMAL );

    oStyle.SetFont( oFont );
    m_poLogTextCtrl->SetDefaultStyle( oStyle );

    // Redirect logging to text control.
    int nMaxLogLevel= poConfig->Read( wxT( "MaxLogLevel" ), 2 );
    new wxChainLogTextCtrl( m_poLogTextCtrl, nMaxLogLevel );
#if __DEBUG__
    ::wxLogMessage( wxT( "%s %s" ), cstrAppName.c_str(), cstrVersionString.c_str() );
    ::wxLogMessage( wxT( "%s" ), cstrAppDescription.c_str() );
    ::wxLogMessage( wxT( "%s" ), cstrStatusCopyRight.c_str() );
    ::wxLogMessage( wxT( "----------------------------------------------------------" ) );
#endif

    // Restore the standard resolution.
    m_nResolution= poConfig->Read( wxT( "Resolution" ), 300 );

    // Restore profile Settings
    int nProfileIndex= poConfig->Read( wxT( "ProfileIndex" ), (long)0 );

    for( int i= 0; i < 999; i++ )
    {
        wxString strMenuItemText;

        if( !poConfig->Exists( wxString::Format( wxT( "/Profile/%d" ), i ) ) )
        {
            break;
        }

        wxIScanImageProfile oProfile( i, m_nResolution );

        if( i <= 9 )
        {
            strMenuItemText= wxString::Format( wxT( "&%d " ), i ) + oProfile.GetName() + wxString::Format( wxT( "\tCtrl+%d " ), i );
        }
        else
        {
            strMenuItemText= wxString::Format( wxT( "%d " ), i ) + oProfile.GetName();
        }

        int nMenuID= ID_DOCUMENTPROFILE0 + i;

        poMenuDocumentProfiles->AppendCheckItem( nMenuID, strMenuItemText );
        Connect( nMenuID, wxEVT_COMMAND_MENU_SELECTED, (wxObjectEventFunction)&wxIScanFrame::OnDocumentProfilesChooseProfile );
    }
    SetProfile( nProfileIndex );

    // Reset file history.
    poConfig->SetPath( wxT( "/Files" ) );
    m_oFileHistory.Load( *poConfig );

    // Delete file history entries of files that do not exist.
    for( int i= m_oFileHistory.GetCount() - 1; i >= 0; i-- )
    {
        wxString strFileName= m_oFileHistory.GetHistoryFile( i );

        if( !::wxFileExists( strFileName ) )
        {
            m_oFileHistory.RemoveFileFromHistory( i );
        }
    }

    // Add file history to the 'files' menu.
    m_oFileHistory.AddFilesToMenu( poMenuFile );
    m_oFileHistory.UseMenu( poMenuFile );
}

wxIScanFrame::~wxIScanFrame()
{
#if __WXPDFDOC__
    // Destroy pending PDF documents.
    if( m_poPdfDoc )
    {
        delete m_poPdfDoc;
    }
#endif // __WXPDFDOC__

    // Save configuration data.
    wxConfigBase *poConfig= wxConfigBase::Get();

    poConfig->SetPath( wxT( "/" ) );
    poConfig->SetPath( cstrwxFrame );

	// Save maximization state.
    poConfig->Write( wxT( "Maximized" ),  (long)IsMaximized() );
    if( IsMaximized() )
    {
        Maximize( false );
    }

	// Save iconifization state.
    if( IsIconized() )
    {
        Iconize( false );
    }

    // Save window position and size.
	//
    int x, y, w, h;

    GetSize( &w, &h );
    GetPosition( &x, &y );
    poConfig->Write( wxT( "x" ), (long)x );
    poConfig->Write( wxT( "y" ), (long)y );
    poConfig->Write( wxT( "w" ), (long)w );
    poConfig->Write( wxT( "h" ), (long)h );

    // Save the sash position.
    poConfig->Write( wxT( "HSashPos" ), (long)m_poHSplitterWindow->GetSashPosition() );

    // Save the resolution as standard resolution.
    poConfig->Write( wxT( "Resolution" ), (long)m_nResolution );

    // ...
    poConfig->Write( wxT( "Fit" ), (long)m_nFit );

    // ...
    poConfig->Write( wxT( "OCR" ), (long)m_bOCR );
    poConfig->Write( wxT( "CmdConvert" ), m_strCmdConvert );
    poConfig->Write( wxT( "CmdOcr" ), m_strCmdOcr );
    poConfig->Write( wxT( "CmdCleanUp" ), m_strCmdCleanUp );

    // ...
    poConfig->Write( wxT( "ProfileIndex" ), m_nProfileIndex );

    // Save file history to config file.
    poConfig->SetPath( wxT( "/Files" ) );
    m_oFileHistory.Save( *poConfig );

#if __DEBUG__
    // Save the log to a text file.
    wxFileName oFileName( wxStandardPaths::Get().GetDocumentsDir(), ::wxGetApp().GetAppName(), wxT( "log" ) );

    if( m_poLogTextCtrl )
    {
        m_poLogTextCtrl->SaveFile( oFileName.GetFullPath() );
    }
#endif // __DEBUG__

    //(*Destroy(wxIScanFrame)
    //*)
}

void wxIScanFrame::OnClose( wxCloseEvent& oEvent )
{
    // Check the SQL edit control if the content is "dirty" and veto closing
    // (if it is not forbidden).
    if( CanClose( oEvent.CanVeto() ) )
    {
        this->Destroy();
    }
    else
    {
        if( oEvent.CanVeto() )
        {
            oEvent.Veto();
        }
        else
        {
            this->Destroy();
        }
    }
}

void wxIScanFrame::OnFileDrop( wxExtFileDropEvent &oEvent )
{
    // Open file.
    FileOpen( oEvent.GetFileName() );
}

void wxIScanFrame::OnFileScanImage( wxCommandEvent& event )
{
    // Show hourglass cursor.
    wxBusyCursor oWait;

    // Object representing the scanner.
    wxScan oScanner;

    // Check if the scanner object initializsation was successful.
    if( !oScanner.IsOK() )
    {
        ::wxLogError( _( "Cannot initialize SANE object." ) );
        return;
    }

    // Check if the sanning device is accessable.
    if( !oScanner.IsDeviceAvailable() )
    {
        ::wxLogError( _( "SANE device not available." ) );
        return;
    }

    // Scan an image, ...
    if( !oScanner.ScanImage( m_oImage ) )
    {
        ::wxLogError( _( "An error occured while scanning." ) );
        return;
    }

    // ... and get its resolution, ...
    m_nResolution= oScanner.GetResolution();

    // ... apply an image profile (crop, convert to monochrome etc.), and ...
    ApplyProfile();

    // ... get the image orientation, and ...
    m_bPortrait= ( m_oImage.GetHeight() > m_oImage.GetWidth() );

    // ... show the image on the canvas window.
    SetBitmap();

    // Reset the image count, the page number and the image filename.
    m_nImageCount= 1;
    m_nPage= 0;
    m_strImageFileName=  _( "unknown.jpg" );

    // Enable (additional) menu options after the 1st scan.
    EnableControls();
}

void wxIScanFrame::OnFileOpen( wxCommandEvent& event )
{
    // Open an "Open file" dialog an let the user choose a filename.
    wxFileDialog oDlg( this,
                       _( "Open file" ),
                       wxT( "" ),
                       wxT( "" ),
                       ::wxGetApp().GetImageFilter(),
                       wxOPEN | wxFILE_MUST_EXIST | wxCHANGE_DIR );

    if( oDlg.ShowModal() == wxID_OK )
    {
        // Show hourglass cursor.
        wxBusyCursor oWait;

        // Open file.
        FileOpen( oDlg.GetPath() );
    }
}

void wxIScanFrame::OnFileSaveAs( wxCommandEvent& event )
{
    // Open a "Save file" dialog and let the user choose a filename.
    wxFileDialog oDlg( this,
                       _( "Save as" ),
                       wxT( "" ),
                       m_strImageFileName,
                       ::wxGetApp().GetImageFilter(),
                       wxSAVE | wxOVERWRITE_PROMPT | wxCHANGE_DIR );

    if( oDlg.ShowModal() == wxID_OK )
    {
        // Show hourglass cursor.
        wxBusyCursor oWait;

        // Save file.
        m_oImage.SaveFile( oDlg.GetPath() );
    }
}

void wxIScanFrame::OnFilePrint( wxCommandEvent & WXUNUSED( oEvent ) )
{
    AdjustOrientation();

    // Data object representing the print dialog's values.
    wxPrintDialogData oPrintDialogData( m_oPrintData );

    // Set the printer dialog's maximal page.
    oPrintDialogData.SetMaxPage( m_nImageCount );

    // Printer object.
    wxPrinter oPrinter( &oPrintDialogData );

    // ...
    wxIViewPrintout oPrintout( this, m_strImageFileName, false, m_nResolution, m_nResolution );

    // Create cancel dialog.
    oPrinter.CreateAbortWindow( this, &oPrintout );

    // Print.
    if( oPrinter.Print( this, &oPrintout, true ) )
    {
        m_oPrintData= oPrinter.GetPrintDialogData().GetPrintData();
    }
    else
    {
        if( wxPrinter::GetLastError() == wxPRINTER_ERROR )
        {
            ::wxLogError( _( "An error occured while printing." ) );
        }
    }

    // Destory the print image (for to save MBs memory).
    m_oPrintImage.Destroy();
}

void wxIScanFrame::OnFilePrintPreview( wxCommandEvent & WXUNUSED( oEvent ) )
{
    AdjustOrientation();

    wxPrintDialogData oPrintDialogData( m_oPrintData );

    // Pass two printout objects: for preview, and possible printing.
    wxPrintPreview *poPreview = new wxPrintPreview(
        new wxIViewPrintout( this, m_strImageFileName, false, m_nResolution, m_nResolution ),
        new wxIViewPrintout( this, m_strImageFileName, false, m_nResolution, m_nResolution ),
        &oPrintDialogData );

    if ( !poPreview || !poPreview->Ok() )
    {
        if( poPreview )
        {
            delete poPreview;
        }
        ::wxLogError( _( "An error occured while creating print preview object.") );
        return;
    }

    // Set the zoom factor to 100%.
    poPreview->SetZoom( 100 );

    int x, y, w, h;

	// Get the frame window position and size and create a print preview
	// window with the same postion and size.
    GetSize( &w, &h );
    GetPosition( &x, &y );
    wxIViewPreviewFrame *poPreviewFrame= new wxIViewPreviewFrame(
                                          poPreview,
                                          this,
                                          _( "Print Preview" ),
                                          wxPoint( x, y ),
                                          wxSize( w, h ) );
    if( !poPreviewFrame )
    {
        delete poPreview;
        ::wxLogError( _( "An error occured while creating print preview window.") );
        return;
    }
    poPreviewFrame->Initialize();
    poPreviewFrame->Show();

    // Destory the print image (for to save MBs memory).
    m_oPrintImage.Destroy();
}

void wxIScanFrame::OnFilePrintSetup( wxCommandEvent & WXUNUSED( oEvent ) )
{
    AdjustOrientation();

    wxPageSetupDialogData oPageSetupData( m_oPrintData );

    oPageSetupData.EnableOrientation( false );

    wxPageSetupDialog oPageSetupDialog( this, &oPageSetupData );

    oPageSetupDialog.ShowModal();
    m_oPrintData= oPageSetupDialog.GetPageSetupData().GetPrintData();
}

void wxIScanFrame::OnFileQuit( wxCommandEvent& event )
{
    Close();
}

void wxIScanFrame::OnMRUFiles( wxCommandEvent &oEvent )
{
	// Get the filename from history menu index,...
    wxString strFilename = m_oFileHistory.GetHistoryFile( oEvent.GetId() - wxID_FILE1 );

	// ... check if it is valid, and...
    if( !strFilename.IsEmpty() )
    {
		// ... open the image file again.
        FileOpen( strFilename );
    }
}

void wxIScanFrame::OnPdfBeginPdf( wxCommandEvent& event )
{
#if defined( __WXPDFDOC__ ) && __WXPDFDOC__ >= 1
    if( !CanClose() )
    {
        return;
    }

    if( !CreateNewPdfDocument() )
    {
        return;
    }

    // Enable (additional) menu options.
    EnableControls();
#else
    ::wxLogError( _( "PDF support is not enabled." ) );
#endif
}

void wxIScanFrame::OnPdfAddPage( wxCommandEvent& event )
{
#if defined( __WXPDFDOC__ ) && __WXPDFDOC__ >= 1
    bool bAddAllPages= false;

    // ...
    if( ( m_nImageCount > 1 ) && ( m_nPage == 0 ) )
    {

        switch( ::wxMessageBox( _( "This is a multipage image. Do you want to add all image pages to the PDF file?" ),
                                ::wxGetApp().GetAppName(),
                                wxYES_DEFAULT | wxNO | wxCANCEL | wxICON_QUESTION  ) )
        {
            case wxYES    : bAddAllPages= true;
                            break;
            case wxCANCEL : return;
        }
    }

    // Show busy cursor (until the automatic variable is destroyed).
    wxBusyCursor oWait;

    // Add page to PDF document.
    AddPdfPage();

    if( bAddAllPages )
    {
        while( m_nPage < ( m_nImageCount - 1 ) )
        {
            // Load next image page.
            OnDocumentNextPage( event );

            // Add page to PDF document.
            AddPdfPage();
        }
    }

    // Enable (additional) menu options.
    EnableControls();
#else
    ::wxLogError( _( "PDF support is not enabled." ) );
#endif
}

void wxIScanFrame::OnPdfSavePdf( wxCommandEvent& event )
{
#if defined( __WXPDFDOC__ ) && __WXPDFDOC__ >= 1
    if( !m_poPdfDoc )
    {
        ::wxLogError( _( "wxPdfDocument is NULL" ) );
        return;
    }

    // Open a "Save file" dialog and let the user choose a filename.
    wxFileDialog oDlg( this,
                       _( "Save as" ),
                       wxT( "" ),
                       _( "unknown.pdf" ),
                       _( "PDF file" ) + wxString( wxT( " (*.pdf)|*.pdf|" ) )
                       + _( "All files" ) + wxString( wxT( "|*.*" ) ),
                       wxSAVE | wxOVERWRITE_PROMPT | wxCHANGE_DIR );

    if( oDlg.ShowModal() == wxID_OK )
    {
        // Show busy cursor (until the automatic variable is destroyed).
        wxBusyCursor oWait;

        // Save file.
        m_poPdfDoc->SaveAsFile( oDlg.GetPath() );

        // Mark PDF document as 'clean'.
        m_bPdfDirty= false;

        // Create a new PDF document.
        OnPdfBeginPdf( event );
    }
#else
    ::wxLogError( _( "PDF support is not enabled." ) );
#endif
}

void wxIScanFrame::OnDocumentFirstPage( wxCommandEvent& WXUNUSED( oEvent ) )
{
    // Show hourglass cursor.
    wxBusyCursor oWait;

    m_nPage= 0;
    m_oImage.LoadFile( m_strImageFileName, wxBITMAP_TYPE_ANY, m_nPage );
    SetBitmap();
m_bPortrait= ( m_oImage.GetHeight() > m_oImage.GetWidth() );
    EnableControls();
}

void wxIScanFrame::OnDocumentPreviousPage( wxCommandEvent& WXUNUSED( oEvent ) )
{
    // Show hourglass cursor.
    wxBusyCursor oWait;

    if( m_nPage > 0 )
    {
        m_nPage--;
        m_oImage.LoadFile( m_strImageFileName, wxBITMAP_TYPE_ANY, m_nPage );
        SetBitmap();
m_bPortrait= ( m_oImage.GetHeight() > m_oImage.GetWidth() );
        EnableControls();
    }
}

void wxIScanFrame::OnDocumentNextPage( wxCommandEvent& WXUNUSED( oEvent ) )
{
    // Show hourglass cursor.
    wxBusyCursor oWait;

    if( m_nPage < ( m_nImageCount - 1 ) )
    {
        m_nPage++;
        m_oImage.LoadFile( m_strImageFileName, wxBITMAP_TYPE_ANY, m_nPage );
        SetBitmap();
m_bPortrait= ( m_oImage.GetHeight() > m_oImage.GetWidth() );
        EnableControls();
    }
}

void wxIScanFrame::OnDocumentLastPage( wxCommandEvent& WXUNUSED( oEvent ) )
{
    // Show hourglass cursor.
    wxBusyCursor oWait;

    m_nPage= m_nImageCount - 1;
    m_oImage.LoadFile( m_strImageFileName, wxBITMAP_TYPE_ANY, m_nPage );
    SetBitmap();
m_bPortrait= ( m_oImage.GetHeight() > m_oImage.GetWidth() );
    EnableControls();
}

void wxIScanFrame::OnDocumentRotate90Left( wxCommandEvent& WXUNUSED( oEvent ) )
{
    // Show hourglass cursor.
    wxBusyCursor oWait;

    // Rotate image and change image orientation.
    m_oImage= m_oImage.Rotate90( false );
    SetBitmap();
    m_bPortrait= !m_bPortrait;
}

void wxIScanFrame::OnDocumentRotate90Right( wxCommandEvent& WXUNUSED( oEvent ) )
{
    // Show hourglass cursor.
    wxBusyCursor oWait;

    // Rotate image and change image orientation.
    m_oImage= m_oImage.Rotate90( true );
    SetBitmap();
    m_bPortrait= !m_bPortrait;
}

void wxIScanFrame::OnDocumentApplyProfile( wxCommandEvent& WXUNUSED( oEvent ) )
{
    // Apply an image profile (crop, convert to monochrome etc.), and ...
    ApplyProfile();

    // ... reset the image on the canvas window.
    SetBitmap();
}

void wxIScanFrame::OnDocumentProfilesChooseProfile( wxCommandEvent& oEvent )
{
    SetProfile( oEvent.GetId() - ID_DOCUMENTPROFILE0 );
}

void wxIScanFrame::OnHelpAbout( wxCommandEvent& event )
{
    wxAboutDialogInfo oAboutDialogInfo;

    oAboutDialogInfo.SetName(        cstrAppName );
    oAboutDialogInfo.SetVersion(     cstrVersionString );
    oAboutDialogInfo.SetDescription( cstrAppDescription );
    oAboutDialogInfo.SetCopyright(   cstrStatusCopyRight );
    oAboutDialogInfo.SetIcon(        wxiscanico_xpm );
    oAboutDialogInfo.SetWebSite(     cstrWebSite );

    ::wxAboutBox( oAboutDialogInfo );
}

void wxIScanFrame::OnCanvasSize( wxIViewCanvasSizeEvent & WXUNUSED( oEvent ) )
{
    SetBitmap();
}

bool wxIScanFrame::CanClose( bool bCanVeto )
{
#if __WXPDFDOC__
    if( IsPdfDirty() )
    {
        switch( ::wxMessageBox( _( "The changes to the current PDF document have not yet been saved. "
                                   "Do you want to save the PDF document now?" ),
                                ::wxGetApp().GetAppName(),
                                wxYES_NO | ( bCanVeto ? wxCANCEL : 0 ) | wxICON_QUESTION ) )
        {
            case wxYES: {
                            wxCommandEvent oDummyEvent( wxEVT_COMMAND_MENU_SELECTED, ID_PDFSAVEASPDF );

                            OnPdfSavePdf  ( oDummyEvent );
                        }
                        return !IsPdfDirty();
            case wxNO : return true;
            default:    return false;
        }
    }
    else
    {
        return true;
    }
#else
    return true;
#endif
}

bool wxIScanFrame::FileOpen( const wxString &strFileName )
{
    // Save the filename.
    wxFileName oFileName( strFileName );
    oFileName.Normalize();
    m_strImageFileName= oFileName.GetFullPath();

    // Open file.
    if( !m_oImage.LoadFile( m_strImageFileName ) )
    {
        // Signal error;
        return false;
    }

    // Get the image count and reset the page number.
    m_nImageCount= m_oImage.GetImageCount( m_strImageFileName );
    m_nPage= 0;

    // Get the image resolution (or use the last acquired one by not
    // changing the resolution).
    if( m_oImage.HasOption( wxIMAGE_OPTION_RESOLUTION ) )
    {
        m_nResolution= m_oImage.GetOptionInt( wxIMAGE_OPTION_RESOLUTION );
    }
    else if( m_oImage.HasOption( wxIMAGE_OPTION_RESOLUTIONX ) )
    {
        m_nResolution= m_oImage.GetOptionInt( wxIMAGE_OPTION_RESOLUTIONX );
    }
    else if( m_oImage.HasOption( wxIMAGE_OPTION_RESOLUTIONY ) )
    {
        m_nResolution= m_oImage.GetOptionInt( wxIMAGE_OPTION_RESOLUTIONY );
    }

    // Get the image orientation.
    m_bPortrait= ( m_oImage.GetHeight() > m_oImage.GetWidth() );

    // Show the image on the canvas window.
    SetBitmap();

    // Enable (additional) menu options after the 1st scan.
    EnableControls();

    // Add filename to file history.
    m_oFileHistory.AddFileToHistory( m_strImageFileName );

    // Signal success.
    return true;
}

void wxIScanFrame::SetBitmap()
{
    // Show busy cursor (until the automatic variable is destroyed).
    wxBusyCursor oWait;

    // Resize image (if necessary).
    switch( m_nFit )
    {
        case wxTIXFITWIDTH  : {
                                   int x, nDummy;

                                   m_poCanvas->GetClientSize( &x, &nDummy );

                                   double xfactor= (double)x / (double)m_oImage.GetWidth();

                                   m_poCanvas->SetBitmap( m_oImage.Scale( (int)( (double)m_oImage.GetWidth()  * xfactor ),
                                                                          (int)( (double)m_oImage.GetHeight() * xfactor ),
                                                                          wxIMAGE_QUALITY_HIGH ) );
                               }
                               break;
        case wxTIXFITWINDOW  : {
                                   int x, y;

                                   m_poCanvas->GetClientSize( &x, &y );

                                   double xfactor= (double)x / (double)m_oImage.GetWidth();
                                   double yfactor= (double)y / (double)m_oImage.GetHeight();

                                   if( xfactor > yfactor )
                                   {
                                       xfactor= yfactor;
                                   }
                                   m_poCanvas->SetBitmap( m_oImage.Scale( (int)( (double)m_oImage.GetWidth()  * xfactor ),
                                                                          (int)( (double)m_oImage.GetHeight() * xfactor ),
                                                                          wxIMAGE_QUALITY_HIGH ) );
                               }
                               break;
        case wxTIXFULLSIZE   : // Nothing to to for this is the default.
        default              : m_poCanvas->SetBitmap( m_oImage );
                               break;
    }
}

void wxIScanFrame::SetProfile( int nProfileIndex )
{
    GetMenuBar()->Check( ID_DOCUMENTPROFILE0 + m_nProfileIndex, false );
    m_nProfileIndex= nProfileIndex;
    GetMenuBar()->Check( ID_DOCUMENTPROFILE0 + m_nProfileIndex, true  );
}

void wxIScanFrame::ApplyProfile()
{
    // Show hourglass cursor.
    wxBusyCursor oWait;

    // Load image conversion profile, and ...
    wxIScanImageProfile oImageProfile( m_nProfileIndex, m_nResolution );

    // ... apply the conversions.
    m_oImage= oImageProfile.ApplyToImage( m_oImage );
}

wxString wxIScanFrame::OCR( const wxString &strTempFileName )
{
    // ...
    wxString strCleanUpCommand= wxString::Format( m_strCmdCleanUp,
                                                  strTempFileName.c_str(),
                                                  strTempFileName.c_str() );
#if __DEBUG__
    ::wxLogMessage( wxT( "wxIScanFrame::OCR - strCleanUpCommand == '%s'" ), strCleanUpCommand.c_str() );
#endif

    wxString strCommand= wxString::Format( m_strCmdConvert,
                                           strTempFileName.c_str(),
                                           strTempFileName.c_str() );
#if __DEBUG__
    ::wxLogMessage( wxT( "wxIScanFrame::OCR - strCommand == '%s'" ), strCommand.c_str() );
#endif
    if( ::wxExecute( strCommand, wxEXEC_SYNC ) != 0 )
    {
        ::wxLogMessage( wxString( wxT( "wxIScanFrame::OCR - " ) )
                        + _( "Creating monochrome tiff file using 'convert' failed." ) );
        ::wxExecute( strCleanUpCommand, wxEXEC_SYNC );
        return wxEmptyString;
    }

    // ...
    strCommand= wxString::Format( m_strCmdOcr,
                                  strTempFileName.c_str(),
                                  strTempFileName.c_str() );
#if __DEBUG__
    ::wxLogMessage( wxT( "wxIScanFrame::OCR - strCommand == '%s'" ), strCommand.c_str() );
#endif
    if( ::wxExecute( strCommand, wxEXEC_SYNC ) != 0 )
    {
        ::wxLogMessage( wxString( wxT( "wxIScanFrame::OCR - " ) )
                        + _( "OCR using 'tesseract' failed." ) );
        ::wxExecute( strCleanUpCommand, wxEXEC_SYNC );
;        return wxEmptyString;
    }

    // ...
    ::wxExecute( strCleanUpCommand, wxEXEC_SYNC );

    // ...
    return strTempFileName + wxT( ".txt" ); // TODO
}

#if __WXPDFDOC__
bool wxIScanFrame::CreateNewPdfDocument()
{
    // Delete any old PDF documents.
    if( m_poPdfDoc  )
    {
        delete m_poPdfDoc;
    }
    SetStatusText( wxString( wxT( "PDF: " ) ) + _( "empty" ), 1 );

    // Create a new PDF document in portrait mode on A4 pager using mm
    // as measurement unit.
    m_poPdfDoc= new wxPdfDocument( wxPORTRAIT,
                                   wxString( _T( "mm" ) ),
                                   wxPAPER_A4 );
    m_nPdfPages= 0;
    m_bPdfDirty= false;
    if( !m_poPdfDoc )
    {
        ::wxLogError( _( "Cannot create wxPdfDocument." ) );
    }
    m_poPdfDoc->SetCreator( ::wxGetApp().GetAppName() );
    m_poPdfDoc->SetAutoPageBreak( false );
    m_poPdfDoc->SetCompression( true );
    return true;
}

bool wxIScanFrame::AddPdfPage()
{
    if( !m_poPdfDoc )
    {
        ::wxLogError( _( "wxPdfDocument is NULL" ) );
        return false;
    }

    // Save image in a temporary JPEG file.
    wxString strTempFileName= wxFileName::CreateTempFileName( wxT( "" ) );

    m_oImage.SaveFile( strTempFileName, wxBITMAP_TYPE_JPEG );

#if defined( __WXSCANDEBUG__ ) && __WXSCANDEBUG__ >= 1
    ::wxLogMessage( wxT( "PDF PARAMETERS" ) );
    ::wxLogMessage( wxT( "\tGetPageWidth == %.2f mm (%f)"  ), m_poPdfDoc->GetPageWidth(),  m_poPdfDoc->GetPageWidth()  / m_oImage.GetWidth() );
    ::wxLogMessage( wxT( "\tGetPageHeight == %.2f mm (%f)" ), m_poPdfDoc->GetPageHeight(), m_poPdfDoc->GetPageHeight() / m_oImage.GetHeight() );
    ::wxLogMessage( wxT( "\tGetImageScale == %f (new: %f)" ), m_poPdfDoc->GetImageScale(), (double)m_nResolution / 72.0 );
    ::wxLogMessage( wxT( "\tGetScaleFactor == %f" ), m_poPdfDoc->GetScaleFactor() );
    ::wxLogMessage( wxT( "\tm_nResolution == %.2f DPI (%.2f/mm)" ), (double)m_nResolution, (double)m_nResolution / 25.4 );
    ::wxLogMessage( wxT( "============" ) );
    ::wxSafeYield();
#endif

    // Add a page, ...
    m_poPdfDoc->AddPage( m_bPortrait ? wxPORTRAIT : wxLANDSCAPE );

    // do some OCR and include the recognized text if enabled, and ...
    if( m_bOCR )
    {
        wxString strOcrTextFileName= OCR( strTempFileName );

        if( !strOcrTextFileName.IsEmpty() )
        {
            wxTextFile oOcrTextFile( strOcrTextFileName );

            if( oOcrTextFile.Open( wxConvUTF8 ) )
            {
                m_poPdfDoc->SetFont( wxT( "Times" ), wxT( "" ) ,8 );
                m_poPdfDoc->SetTextColor( 0.0, 0.0, 0.0, 0.0 );

                for( wxString strTextLine= oOcrTextFile.GetFirstLine();
                     !oOcrTextFile.Eof();
                     strTextLine = oOcrTextFile.GetNextLine() )
                {
#if __DEBUG__
                    ::wxLogMessage( strTextLine );
#endif
                    m_poPdfDoc->Cell( 0, 3, strTextLine, wxPDF_BORDER_NONE, 1 );
                }
            }

            // ...
            ::wxRemoveFile( strOcrTextFileName );
        }
    }

    // ... put the image onto the page.
    m_poPdfDoc->SetImageScale(  (double)m_nResolution / 72.0 );
    m_poPdfDoc->Image( strTempFileName, 0 , 0, 0, 0, wxT( "image/jpeg" ) );

    // Remove temporary file.
    ::wxRemoveFile( strTempFileName );

    // Incremement the page counter and mark the PDF document as 'dirty'.;
    m_nPdfPages++;
    m_bPdfDirty= true;
    SetStatusText( wxString( wxT( "PDF: " ) ) + wxString::Format( _("Page %d" ), m_nPdfPages), 1 );

    // Signal success.
    return true;
}
#endif

void wxIScanFrame::EnableControls()
{
    // Enable/disable menu items (on first call).
    bool bImageOk= m_oImage.IsOk();

    GetMenuBar()->Enable(     ID_FILESAVEAS,                bImageOk );
    GetToolBar()->EnableTool( ID_FILESAVEAS_TB,             bImageOk );
    GetMenuBar()->Enable(     ID_FILEPRINT,                 bImageOk );
    GetToolBar()->EnableTool( ID_FILEPRINT_TB,              bImageOk );
    GetMenuBar()->Enable(     ID_FILEPRINTPREVIEW,          bImageOk );
    GetMenuBar()->Enable(     ID_FILEPRINTSETUP,            bImageOk );
#if __WXPDFDOC__
    GetMenuBar()->Enable(     ID_PDFADDPAGE,                bImageOk );
    GetToolBar()->EnableTool( ID_PDFADDPAGE_TB,             bImageOk );
#else
    GetMenuBar()->Enable(     ID_PDFADDPAGE,                false );
    GetToolBar()->EnableTool( ID_PDFADDPAGE_TB,             false );
#endif // __WXPDFDOC__
    GetMenuBar()->Enable(     ID_DOCUMENTROTATE90LEFT,      bImageOk );
    GetMenuBar()->Enable(     ID_DOCUMENTROTATE90RIGHT,     bImageOk );
    GetMenuBar()->Enable(     ID_DOCUMENTAPPLYPROFILE,      bImageOk );

#if __WXPDFDOC__
    // ...
    bool bCanSavePdfFile= ( m_nPdfPages > 0 );

    GetMenuBar()->Enable(     ID_PDFSAVEASPDF,              bCanSavePdfFile );
    GetToolBar()->EnableTool( ID_PDFSAVEASPDF_TB,           bCanSavePdfFile );
#else
    GetMenuBar()->Enable(     ID_PDFSAVEASPDF,              false );
    GetToolBar()->EnableTool( ID_PDFSAVEASPDF_TB,           false );
#endif // __WXPDFDOC__

    // Enable/disable page controls
    bool bNotFirstPage= ( m_nPage > 0 );
    bool bNotLastPage=  ( m_nPage < ( m_nImageCount - 1 ) );

    GetMenuBar()->Enable(     ID_DOCUMENTFIRSTPAGE,         bImageOk && bNotFirstPage );
    GetToolBar()->EnableTool( ID_DOCUMENTFIRSTPAGE_TB,      bImageOk && bNotFirstPage );
    GetMenuBar()->Enable(     ID_DOCUMENTPREVIOUSPAGE,      bImageOk && bNotFirstPage );
    GetToolBar()->EnableTool( ID_DOCUMENTPREVIOUSPAGE_TB,   bImageOk && bNotFirstPage );
    GetMenuBar()->Enable(     ID_DOCUMENTNEXTPAGE,          bImageOk && bNotLastPage  );
    GetToolBar()->EnableTool( ID_DOCUMENTNEXTPAGE_TB,       bImageOk && bNotLastPage  );
    GetMenuBar()->Enable(     ID_DOCUMENTLASTPAGE,          bImageOk && bNotLastPage  );
    GetToolBar()->EnableTool( ID_DOCUMENTLASTPAGE_TB,       bImageOk && bNotLastPage  );
}

// ...
//
// NOTE: This function paints the image while printing or print previewing!
//
bool wxIScanFrame::Paint( wxDC& oDc )
{
    // If there is a bitmap connected to this canvas window (re)paint it.
    if( oDc.Ok() && m_oPrintImage.Ok() )
    {
        // Repaint the bitmap.
        oDc.DrawBitmap( wxBitmap( m_oPrintImage ), 0, 0 );
        return true;
    }
    else
    {
        return false;
    }
}

// ...
//
// NOTE: This function returns the image extents in mm while printing or
//       print previewing!
//
bool wxIScanFrame::GetPageSizeMM( int *pnWidth, int *pnHeight )
{
     if( m_oPrintImage.Ok() )
     {
         (*pnWidth)=  (int)( (double)m_oPrintImage.GetWidth()  / (double)m_nResolution * 25.4 );
         (*pnHeight)= (int)( (double)m_oPrintImage.GetHeight() / (double)m_nResolution * 25.4 );
         return true;
     }
     else
     {
         return false;
     }
}

// ...
//
bool wxIScanFrame::GotoPage( int nPage )
{
    // NOTE: The wxPrintOut object passes pages one-base. We need it zero-
    //       base. Therefore we decrement the page number by 1.
    //
    nPage--;

    // Avoid loading an already loaded image.
    if( nPage == m_nPage )
    {
        m_oPrintImage= m_oImage;
        return true;
    }
    else
    {
        return m_oPrintImage.LoadFile( m_strImageFileName, wxBITMAP_TYPE_ANY, nPage );
    }
}
