/***************************************************************
 * Name:      wxiscanframe.h
 * Purpose:   Defines Application Frame
 * Author:    Daniel Nell (daniel.nell@nellresearch.de)
 * Created:   2008-05-04
 * Copyright: Daniel Nell (www.nellresearch.de)
 * License:
 **************************************************************/

#ifndef WXISCANMAIN_H
#define WXISCANMAIN_H

//(*Headers(wxIScanFrame)
#include <wx/toolbar.h>
#include <wx/menu.h>
#include <wx/scrolwin.h>
#include <wx/splitter.h>
#include <wx/statusbr.h>
#include <wx/frame.h>
#include <wx/textctrl.h>
//*)

// Additional wxWidgets header files
#include <wx/docview.h>

// Private header files.
#include "wxextfiledroptarget.h"
#include "wxiviewcanvas.h"
#include "wxiviewpaint.h"


#if __WXPDFDOC__
    class wxPdfDocument;
#endif // __WXPDFDOC__


class wxIScanFrame: public wxFrame, public wxIViewPaintBase
{
    public:
        enum
        {
            wxTIXFULLSIZE,
            wxTIXFITWIDTH,
            wxTIXFITWINDOW
        };

    public:
        wxIScanFrame(wxWindow* parent,wxWindowID id = -1);
        virtual ~wxIScanFrame();

    public:
        virtual bool CanClose( bool bCanVeto= true );           ///< ...
        virtual wxToolBar *GetToolBar(){ return m_poToolBar; }  ///< ...
        virtual bool FileOpen( const wxString &strFileName );   ///< ...
        virtual void SetBitmap();                               ///< ...
        virtual void SetProfile( int nProfileIndex );           ///< ...
        virtual void ApplyProfile();                            ///< ...
        virtual wxString OCR( const wxString &strTempFileName ); ///< ...
#if __WXPDFDOC__
        virtual bool CreateNewPdfDocument();                    ///< ...
        virtual bool AddPdfPage();                              ///< ...
        virtual bool IsPdfDirty(){ return m_bPdfDirty; }        ///< ...
#endif // __WXPDFDOC__
        virtual void EnableControls();                          ///< ...

    public:
        // Printer handling (introduced by base class wxIViewPaintBase)
        virtual void AdjustOrientation(){ m_oPrintData.SetOrientation( m_bPortrait ? wxPORTRAIT : wxLANDSCAPE ); }
        virtual bool GetPageSizeMM( int *pnWidth, int *pnHeight );
        virtual int  GetMaxPage(){ return m_nImageCount; }
        virtual bool GotoPage( int nPage );
        virtual bool Paint( wxDC& oDc );

    private:
        wxImage m_oImage;                                       ///< Image object.
        wxImage m_oPrintImage;                                  ///< Image object for printing.
        wxString m_strImageFileName;                            ///< ...
        int m_nPage;                                            ///< ...
        int m_nImageCount;                                      ///< ...
        int m_nResolution;                                      ///< ...
        int m_bPortrait;                                        ///< ...
        int m_nFit;                                             ///< ...
        bool m_bOCR;                                            ///< ...
        wxString m_strCmdConvert;                               ///< ...
        wxString m_strCmdOcr;                                   ///< ...
        wxString m_strCmdCleanUp;                               ///< ...
        int m_nProfileIndex;                                    ///< ...

        // File history
        wxFileHistory m_oFileHistory;                           ///< History of last opened files

        // Printing support.
        wxPrintData m_oPrintData;                               ///< ...

#if __WXPDFDOC__
        // PDF support
        wxPdfDocument *m_poPdfDoc;                              ///< ...
        int m_nPdfPages;                                        ///< ...
        bool m_bPdfDirty;                                       ///< ...
#endif // __WXPDFDOC__

    private:
        //(*Handlers(wxIScanFrame)
        void OnFileQuit(wxCommandEvent& event);
        void OnFileScanImage(wxCommandEvent& event);
        void OnFileOpen(wxCommandEvent& event);
        void OnFileSaveAs(wxCommandEvent& event);
        void OnFilePrint( wxCommandEvent &oEvent );             ///< Handle menu command "File|Print".
        void OnFilePrintPreview( wxCommandEvent &oEvent );      ///< Handle menu command "File|Print Preview".
        void OnFilePrintSetup( wxCommandEvent &oEvent );        ///< Handle menu command "File|Page Setup ".
        void OnPdfBeginPdf(wxCommandEvent& event);
        void OnPdfAddPage(wxCommandEvent& event);
        void OnPdfSavePdf(wxCommandEvent& event);
        void OnDocumentFirstPage(wxCommandEvent& event);
        void OnDocumentPreviousPage(wxCommandEvent& event);
        void OnDocumentNextPage(wxCommandEvent& event);
        void OnDocumentLastPage(wxCommandEvent& event);
        void OnDocumentRotate90Left(wxCommandEvent& event);
        void OnDocumentRotate90Right(wxCommandEvent& event);
        void OnHelpAbout(wxCommandEvent& event);
        void OnClose(wxCloseEvent& event);
        void OnDocumentApplyProfile(wxCommandEvent& event);
        //*)
        void OnDocumentProfilesChooseProfile(wxCommandEvent &oEvent);
        void OnMRUFiles( wxCommandEvent &oEvent );              ///< Handle menu command "File|[MOST RECENTLY USES FILES]".
        void OnFileDrop( wxExtFileDropEvent &oEvent );          ///< ...
        void OnCanvasSize( wxIViewCanvasSizeEvent &oEvent );    ///< ...

        //(*Identifiers(wxIScanFrame)
        static const long ID_CANVAS;
        static const long ID_LOGTEXTCTRL;
        static const long ID_HSPLITTERWINDOW;
        static const long ID_FILESCANIMAGE;
        static const long ID_FILEOPEN;
        static const long ID_FILESAVEAS;
        static const long ID_FILEPRINT;
        static const long ID_FILEPRINTPREVIEW;
        static const long ID_FILEPRINTSETUP;
        static const long ID_FILEQUIT;
        static const long ID_DOCUMENTFIRSTPAGE;
        static const long ID_DOCUMENTPREVIOUSPAGE;
        static const long ID_DOCUMENTNEXTPAGE;
        static const long ID_DOCUMENTLASTPAGE;
        static const long ID_DOCUMENTROTATE90LEFT;
        static const long ID_DOCUMENTROTATE90RIGHT;
        static const long ID_DOCUMENTAPPLYPROFILE;
        static const long ID_MENUDOCUMENTPROFILES;
        static const long ID_PDFBEGINPDF;
        static const long ID_PDFADDPAGE;
        static const long ID_PDFSAVEASPDF;
        static const long ID_HELPABOUT;
        static const long ID_STATUSBAR;
        static const long ID_FILESCANIMAGE_TB;
        static const long ID_FILEOPEN_TB;
        static const long ID_FILESAVEAS_TB;
        static const long ID_PDFBEGINPDF_TB;
        static const long ID_PDFADDPAGE_TB;
        static const long ID_PDFSAVEASPDF_TB;
        static const long ID_DOCUMENTFIRSTPAGE_TB;
        static const long ID_DOCUMENTPREVIOUSPAGE_TB;
        static const long ID_DOCUMENTNEXTPAGE_TB;
        static const long ID_DOCUMENTLASTPAGE_TB;
        static const long ID_FILEPRINT_TB;
        static const long ID_TOOLBAR;
        //*)
        static const long ID_DOCUMENTPROFILE0;


        //(*Declarations(wxIScanFrame)
        wxIViewCanvas* m_poCanvas;
        wxStatusBar* m_poStatusBar;
        wxToolBar* m_poToolBar;
        wxSplitterWindow* m_poHSplitterWindow;
        wxTextCtrl* m_poLogTextCtrl;
        //*)

        DECLARE_EVENT_TABLE()
};

#endif // WXISCANMAIN_H
