/***************************************************************
 * Name:      wxiscanframe.h
 * Purpose:   Defines SANE Wrapper for wxWidgets
 * Author:    Daniel Nell (daniel.nell@nellresearch.de)
 * Created:   2008-05-04
 * Copyright: Daniel Nell (www.nellresearch.de)
 * License:   wxWindows license
**************************************************************/

#ifndef WXSCAN_H_INCLUDED
#define WXSCAN_H_INCLUDED

// Include additional headers.
#include <sane/sane.h>
#include <wx/arrstr.h>

// Define some convenience macros.
#define SANE_STRING_TO_WXSTRING( SaneString )                                                               \
    wxString( (const char *)SaneString, wxConvISO8859_1, wxSTRING_MAXLEN )

#define GET_DEVICE_PARAM( PARAM )                                                                           \
    if( nIndex < 0 )                                                                                        \
    {                                                                                                       \
        nIndex= m_nDeviceIndex;                                                                             \
    }                                                                                                       \
    if( ( nIndex >= 0 ) && ( nIndex < m_nDeviceCount ) )                                                    \
    {                                                                                                       \
        return SANE_STRING_TO_WXSTRING( m_ppDeviceList[nIndex]->PARAM );                                    \
    }                                                                                                       \
    return wxEmptyString;

// ...
#ifdef __UNIX_LIKE__
#   define wxScan  wxScanSane
#endif

// ...
class wxScanBase
{
  public:
    /// \brief Standard constructor
    ///
    /// ...
    wxScanBase(){}

    /// Virtual destructor
    virtual ~wxScanBase(){}

    /// ...
    virtual bool IsOK() = 0;
    virtual bool IsDeviceAvailable() = 0;
    virtual int GetResolution() = 0;

    /// ...
    virtual bool ScanImage( wxImage &oImage ) = 0;
};

// ...
class wxScanSane : public wxScanBase
{
  public:
    /// \brief Standard constructor
    ///
    /// ...
    wxScanSane( bool bOnlyLocalDevices= false );

    /// Virtual destructor.
    virtual ~wxScanSane();

    /// ...
    virtual bool IsOK(){ return m_SaneStatus == SANE_STATUS_GOOD ? true : false; }
    virtual bool IsDeviceAvailable(){ return ( m_nDeviceCount <= 0 ) || ( m_nDeviceIndex < 0 ) || ( m_nDeviceIndex >= m_nDeviceCount ) ? false : true; }

    /// ...
    virtual bool ScanImage( wxImage &oImage );

    /// ...
    virtual bool SetDeviceIndex( int nIndex );
    virtual SANE_Status GetSaneStatus(){ return m_SaneStatus; }
    virtual wxString GetSaneStatusString(){ return SANE_STRING_TO_WXSTRING( sane_strstatus( GetSaneStatus() ) ); }
    virtual int GetResolution(){ return m_nResolution; }
    virtual bool SaneGetDevices();
    virtual wxString GetDeviceName( int nIndex= -1 ){ GET_DEVICE_PARAM( name ) }
    virtual wxString GetDeviceVendor( int nIndex= -1 ){ GET_DEVICE_PARAM( vendor ) }
    virtual wxString GetDeviceModel( int nIndex= -1 ){ GET_DEVICE_PARAM( model ) }
    virtual wxString GetDeviceType( int nIndex= -1 ){ GET_DEVICE_PARAM( type ) }

  private:
    /// ...
    SANE_Status m_SaneStatus;

    /// ...
    bool m_bOnlyLocalDevices;

    /// ...
    const SANE_Device **m_ppDeviceList;

    /// ...
    int m_nDeviceCount;

    /// ...
    int m_nDeviceIndex;

    /// ...
    int m_nResolution;
};

#endif // WXSCAN_H_INCLUDED
